/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('user-profile-entity-list', function () {

  juju.components.UserProfileEntityList = React.createClass({
    // broadcastStatus is necessary for communicating loading status back to
    // the parent SectionLoadWatcher.
    propTypes: {
      broadcastStatus: React.PropTypes.func,
      changeState: React.PropTypes.func.isRequired,
      charmstore: React.PropTypes.object.isRequired,
      getDiagramURL: React.PropTypes.func.isRequired,
      type: React.PropTypes.string.isRequired,
      user: React.PropTypes.object,
      users: React.PropTypes.object.isRequired
    },

    getInitialState: function getInitialState() {
      this.xhrs = [];

      return {
        entityList: [],
        loadingEntities: false
      };
    },

    getDefaultProps: function getDefaultProps() {
      // Just in case broadcastStatus isn't passed in (e.g., in tests), calls
      // to it should not fail, so default to an empty function.
      return {
        broadcastStatus: function broadcastStatus() {}
      };
    },

    componentWillMount: function componentWillMount() {
      var users = this.props.users;
      if (users.charmstore && users.charmstore.user) {
        this._fetchEntities(this.props);
      }
    },

    componentWillUnmount: function componentWillUnmount() {
      this.xhrs.forEach(function (xhr) {
        xhr && xhr.abort && xhr.abort();
      });
    },

    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
      var props = this.props;
      // Compare next and previous charmstore users in a data-safe manner.
      var prevCSUser = props.users.charmstore || {};
      var nextCSUser = nextProps.users.charmstore || {};
      if (nextCSUser.user !== prevCSUser.user) {
        this._fetchEntities(nextProps);
      }
    },

    /**
      Requests a list from charmstore of the user's entities.
       @method _fetchEntities
      @param {Object} props the component properties to use.
    */
    _fetchEntities: function _fetchEntities(props) {
      var _this = this;

      var callback = this._fetchEntitiesCallback;
      var charmstore = props.charmstore;
      var username = props.users.charmstore && props.users.charmstore.user;
      if (charmstore && charmstore.list && username) {
        this.props.broadcastStatus('starting');
        // Delay the call until after the state change to prevent race
        // conditions.
        this.setState({ loadingEntities: true }, function () {
          var xhr = charmstore.list(username, callback, props.type);
          _this.xhrs.push(xhr);
        });
      }
    },

    /**
      Callback for the request to list a user's entities.
       @method _fetchEntitiesCallback
      @param {String} error The error from the request, or null.
      @param {Object} data The data from the request.
    */
    _fetchEntitiesCallback: function _fetchEntitiesCallback(error, data) {
      var _this2 = this;

      this.setState({ loadingEntities: false }, function () {
        var broadcastStatus = _this2.props.broadcastStatus;
        if (error) {
          broadcastStatus('error');
          console.error('Can not retrieve entities: ', error);
          return;
        }
        if (!data || !data.length || data.length === 0) {
          broadcastStatus('empty');
        } else {
          broadcastStatus('ok');
        }
        _this2.setState({ entityList: data });
      });
    },

    /**
      Generate a list of tags.
       @method _generateTags
      @param {Array} tagList A list of tags.
      @param {String} id The id of the entity.
      @returns {Object} A list of tag components.
    */
    _generateTags: function _generateTags(tagList, id) {
      if (!tagList) {
        return;
      }
      var tags = [];
      tagList.forEach(function (tag) {
        tags.push(React.createElement(
          'li',
          { className: 'user-profile__comma-item',
            key: id + '-' + tag },
          tag
        ));
      });
      return React.createElement(
        'ul',
        { className: 'user-profile__list-tags' },
        tags
      );
    },

    /**
      Generate a list of series.
       @method _generateSeries
      @param {Array} series A list of series.
      @param {String} id The id of the entity.
      @returns {Object} A list of series components.
    */
    _generateSeries: function _generateSeries(series, id) {
      if (!series) {
        return;
      }
      var listItems = [];
      series.forEach(function (release) {
        listItems.push(React.createElement(
          'li',
          { className: 'user-profile__comma-item',
            key: id + '-' + release },
          release
        ));
      });
      return React.createElement(
        'ul',
        { className: 'user-profile__list-series' },
        listItems
      );
    },

    /**
      Construct the URL for a service icon.
       @method _getIcon
      @param {String} id The service ID.
      @returns {String} The icon URL.
    */
    _getIcon: function _getIcon(id) {
      if (!id) {
        return;
      }
      var cs = this.props.charmstore;
      var path = id.replace('cs:', '');
      return cs.url + '/' + path + '/icon.svg';
    },

    /**
      Generate the details for the provided bundle.
       @method _generateBundleRow
      @param {Object} bundle A bundle object.
      @returns {Array} The markup for the row.
    */
    _generateBundleRow: function _generateBundleRow(bundle) {
      var _this3 = this;

      var id = bundle.id;
      var services = [];
      var applications = bundle.applications || bundle.services || {};
      var serviceNames = Object.keys(applications);
      serviceNames.forEach(function (serviceName, idx) {
        var service = applications[serviceName];
        var id = service.charm;
        var key = 'icon-' + idx + '-' + id;
        services.push(React.createElement('img', { className: 'user-profile__list-icon',
          key: key,
          src: _this3._getIcon(id),
          title: service.charm }));
      });
      var unitCount = bundle.unitCount || React.createElement(
        'span',
        null,
        ' '
      );
      return React.createElement(
        juju.components.UserProfileEntity,
        {
          changeState: this.props.changeState,
          entity: bundle,
          getDiagramURL: this.props.getDiagramURL,
          key: id,
          type: 'bundle' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col five-col ' + 'user-profile__list-name' },
          bundle.name,
          this._generateTags(bundle.tags, id)
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col ' + 'user-profile__list-icons' },
          services
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col one-col prepend-one' },
          unitCount
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col last-col' },
          bundle.owner
        )
      );
    },

    /**
      Generate the header for the bundles.
       @method _generateBundleHeader
      @returns {Array} The markup for the header.
    */
    _generateBundleHeader: function _generateBundleHeader() {
      return React.createElement(
        'li',
        { className: 'user-profile__list-header twelve-col' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col five-col' },
          'Name'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col ' + 'user-profile__list-icons' },
          'Charms'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col one-col prepend-one' },
          'Units'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col last-col' },
          'Owner'
        )
      );
    },

    /**
      Generate the details for the provided charm.
       @method _generateCharmRow
      @param {Object} charm A charm object.
      @returns {Array} The markup for the row.
    */
    _generateCharmRow: function _generateCharmRow(charm) {
      var id = charm.id;
      // Ensure the icon is set.
      charm.icon = charm.icon || this._getIcon(id);
      return React.createElement(
        juju.components.UserProfileEntity,
        {
          changeState: this.props.changeState,
          entity: charm,
          key: id,
          type: 'charm' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col ' + 'user-profile__list-name' },
          charm.name,
          this._generateTags(charm.tags, id)
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col four-col' },
          this._generateSeries(charm.series, id)
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col one-col ' + 'user-profile__list-icons' },
          React.createElement('img', { className: 'user-profile__list-icon',
            src: charm.icon,
            title: charm.name })
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col ' + 'prepend-two last-col' },
          charm.owner
        )
      );
    },

    /**
      Generate the header for the charms.
       @method _generateCharmHeader
      @returns {Array} The markup for the header.
    */
    _generateCharmHeader: function _generateCharmHeader() {
      return React.createElement(
        'li',
        { className: 'user-profile__list-header twelve-col' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col' },
          'Name'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col seven-col' },
          'Series'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col last-col' },
          'Owner'
        )
      );
    },

    render: function render() {
      var type = this.props.type;
      var classes = classNames('user-profile__' + type + '-list', { 'twelve-col': this.state.loadingEntities });
      if (this.state.loadingEntities) {
        return React.createElement(
          'div',
          { className: classes },
          React.createElement(juju.components.Spinner, null)
        );
      }
      var list = this.state.entityList;
      if (!list || list.length === 0) {
        return null;
      }
      var generateRow = void 0,
          header = void 0,
          title = void 0;
      if (type === 'bundle') {
        generateRow = this._generateBundleRow;
        header = this._generateBundleHeader();
        title = 'Bundles';
      } else if (type === 'charm') {
        generateRow = this._generateCharmRow;
        header = this._generateCharmHeader();
        title = 'Charms';
      }
      var rows = list.map(generateRow);
      return React.createElement(
        'div',
        { className: classes },
        React.createElement(
          'div',
          { className: 'user-profile__header twelve-col no-margin-bottom' },
          title,
          React.createElement(
            'span',
            { className: 'user-profile__size' },
            '(',
            list.length,
            ')'
          )
        ),
        React.createElement(
          'ul',
          { className: 'user-profile__list twelve-col' },
          header,
          rows
        )
      );
    }

  });
}, '', {
  requires: ['loading-spinner', 'user-profile-entity']
});